 ;=============================================================================================
 ;=============================================================================================
 ;  PROGRAM FOR SEVEN SEGMENT ELECTROMAGNETIC DISPLAY (EMD) USING ATtiny2313 MICROCONTROLLER
 ;=============================================================================================
 ;=============================================================================================

 ; Date: 4/15/2018 
 ; File: EMDisplayTiny19Article.asm 
 ; Written by: Dave Goodsell
 ;             dsgoodsell@verizon.net 
 
;==============================================================================
;                         			INTRODUCTION
;==============================================================================
;This program was written to support a larger Telephone Based project and may
;need to be adapted for other uses.

;IDP required: Atmel Studio 7 (free) 
;Chip used:
;	Atmel ATTiny2313 8 bit Microcontroller
;	2K ISP flash memory
;	128 bytes EEPROM
;	128 bytes SRAM
;	32 registers

;Note: During initial programming, the ATTiny2313 assembly must be assigned an address(1-13) which will be stored in its onboard EEPROM.
;	   Step 1: Modify the "ID Number" to the desired value, in the section below entitled USER SELECTED VALUES.
;	   Step 2: Uncomment the ";rcall  WriteIDtoEEPROM" line, in the section below entitled INITIALIZE EEPROM, REGISTERS, TIMERS.
;	   Step 3: Press the Start (programming) pointer button above. The program should now compile and be stored in Flash memory and the EEPROM.
;	   Step 4: Re-Comment Out the line referenced Step 2. 		
;      Step 5: Additional changes to the program can be made and loaded into Flash, while the ID Number will remain unchanged. 

;Basically, the EMD program has two Modes, SXS and SPI.
;SXS Mode: Pulses from a vintage rotary telephone dial are totalized and displayed. A reset pulse blanks the display.
;SPI Mode: A host microcontroller sends 1 byte via two SPI lines to the Tiny. High nib = Address, low nib = Digit.
;          If the Address nib matches the EEPROM, the Digit nib is displayed.
;		   If Digit nib = 0, the display is blanked. If Digit nib = 10, a "0" is displayed. Modify as per your application.	

;Mode Input: Low = SPI mode, high = SXS mode
;
;The telephone project that originally used this display was electrically very noisy and unreliable at using an interrupt to sense when the mode
;was changed. In place of it, a number of "check Mode change" routines have been inserted throughout the program. These may be removed if an  
;interrupt routine is re-instituted or if only one mode is needed. 

;Available Programmers/Debuggers for ATTiny2313:
;	 AVRISP mkII: Obsolete, but available on Ebay for $50. No debugging capability.
;	 ATATMEL-ICE Programmer/Debugger: Available from numerous vendors, $91.
;	 Note: The 3-board EMD unit must be powered from an external 14.5VDC power source, when being programmed.


;==============================================================================
;                         			 INCLUDES
;==============================================================================
;
.nolist
.include "tn2313def.inc"
.list
;

;==============================================================================
;                         		     EQUATES
;==============================================================================
;
 
 .equ segPulseOut	=	PA0		;output	
 .equ ModePinIn		=	PA1		;"mode" input (pulled up internally = high)
 .equ prgm0			=	PA2		;RESET		prgm & SPI?

 .equ Spare1		=	PB0		;output     not used	
 .equ Spare2		=	PB1		;output		not used
 .equ TestLED		=	PB2		;output			
 .equ SXSCountIn	=	PB3		;input			
 .equ SXSReleaseIn	=	PB4		;input
 .equ prgm1			=	PB5		;MOSI 		prgm & SPI DI  MasterOutSlaveIn / Data In
 .equ SPIOutput		=	PB6		;MISO 		prgm           MasterInSlaveOut / Data Out		
 .equ prgm3			=	PB7		;USCK		prgm & SPI Clk Clock In

 .equ segBOut		=	PD0		;output
 .equ segFOut		=	PD1		;output     
 .equ segGOut		=	PD2		;output	
 .equ segCOut		=	PD3		;output	
 .equ segEOut		=	PD4		;output
 .equ segDOut		=	PD5		;output
 .equ segAOut		 =	PD6		;output

 
;==============================================================================
;                             USER SELECTED VALUES 
;==============================================================================
;
;EMD ID:
.equ	IDNumber			=  4		; Enter the desired ID number of the EMD, which will be stored in onboard EEPROM
										; Also see "
;Misc:
.equ	DebounceTime		= 25		; 25 ms of debounce for tactile buttons	
.equ	SegmentPulseWidth	= 25		; 25 ms pulse to STBY pins in motor drivers, 10 ms was not enough to reset seg G
.equ	TwoMsDelayValue		=  2		;  2 ms sampling rate for detecting dial pulses

;==============================================================================
;                             REGISTER DEFINITIONS
;==============================================================================
;
.def	RegZero			=	R0			; reserved

.def	a				=	R1			; unused	
.def	b				=	R2			;    "
.def	c				=	R3			;    "
.def	d				=	R4			;   etc 
.def	e				=	R5			; 
.def	f				=	R6			; 
.def	g				=	R7			; 
.def 	Counter0		=	R8			; long interval loop
.def  	Counter1		=	R9 			; long interval loop
.def  	Counter2  		=	R10			; long interval loop
.def  	h				=	R11			;
.def  	i				=	R12
.def  	Timer0Cntr		=	R13			; caution: used in Timer0 interrupt
.def  	j				=	R14
.def  	k				=   R15
	 
.def  	temp    		=	R16
.def  	temp1   		=	R17
.def	Mode			=	R18
.def	temp2			=	R19
.def  	IDRegister		= 	R20         ;EMD ID register
.def  	RcvdSPIByte	    = 	R21	
.def  	HighIntervalCounter	= 	R22
.def  	PulseCounter 	=	R23			;AKA as MstrCntr
.def  	SpareRegister1	 =	R24
.def  	m				=   R25 

;.def	XL				=	R26			; reserved
;.def	XH				=	R27			; reserved
;.def	YL				=	R28			; reserved
;.def	YH				=	R29			; reserved
;.def	ZL				=	R30			; reserved
;.def	ZH				=	R31			; reserved


;		Flags			=	R18	
.equ	aa				=	0			; unused
.equ	bb				=  	1			; unused
.equ    ModeFlag		=   2			; 
.equ	ResetFlag		=  	3			; 
.equ	dd				=   4			; unused
.equ	ee				=	5			; unused
.equ	ff				=	6			; unused
.equ	gg				=	7			; unused


;==============================================================================
;                      	 RESET AND INTERRUPT VECTORS
;==============================================================================
;
.CSEG									


;==============================================================================
;                     CLEAR RAM, SET STACK POINTER 
;==============================================================================

RESET:

;Clear RAM
;==========
	clr		temp
;0x0060-0x00DE;
	ldi		XL, 0x60
	ldi		XH, 0x00
	ldi		YL, 0xDE
	ldi		YH, 0x00
RAMClrLoop:
	st		X+, temp	
	cp		YL, XL
	brne	RAMClrLoop


;Designate Stack Pointer Location
;=================================
	ldi		temp, 0xDF
	out		SPL, temp

;
;==============================================================================
;                         		 PINS AND PORTS
;==============================================================================
;
; PA0:	out		segPulseOut			[starts low, very important!]		
; PA1:	in		ModePinIn			[ext 10K pullup]	   
; PA2:	in		RESET, Prgm			[ISP header, on-board reset button, ext 10K pullup]	

; PB0:	out		Spare1				
; PB1:	out		Spare2				
; PB2:	out		TestLED 	 		[on-board LED]
; PB3:	in		SXSCountIn			[ext 10k pull up]							
; PB4:	in		SXSReleaseIn		[ext 10k pull up]
; PB5:	in		MOSI/DI, Prgm 		[40K pulled up, ISP header]	    DI is slave SPI data input	
; PB6:	out		MISO/DO,Prgm 		[ISP header only]				DO is slave SPI data output, nut not used by program	
; PB7:	in		USCK,Prgm			[40K pulled up, ISP header]	    USCK is slave SPI clock input			

; PD0:  out     segAOut                
; PD1:  out     segBOut                    
; PD2:  out     segCOut	 
; PD3:  out		segDOut	
; PD4:	out		segEOut
; PD5:	out		segFOut					
; PD6:	out		segGOut					
; PD7:  ---     Not available 
					

;Enable Pin Directions and Pullups (~40K ohms)
;-------------------------------------

	ldi			temp, 0b00000001	;PortA, In and Out pins, 1=out, 0=in
	out			DDRA, temp

	ldi			temp, 0b01000111	;PortB, In and Out pins
	out			DDRB, temp

	ldi			temp, 0b01111111	;PortD, In and Out pins
	out			DDRD, temp

	cbi			PortA, PA0			;PA0, segPulseOut = starts low (super important!)

	ldi			temp, 0b10100000    ;pullups on some input pins = 1
	out			PortB, temp

	ldi			temp, 0b00000000	;segment state pins all start low
	out			PortD, temp



;==============================================================================
;                     INITIALIZE EEPROM, REGISTERS, TIMERS
;==============================================================================

;Write ID Number to EEPROM just ONE TIME during initial programming, then comment out  
;=====================================================================================
    ldi  	temp1, IDNumber			
	;rcall	WriteIDtoEEPROM			;RE-COMMENT OUT THIS LINE AFTER ID NUMBER HAS LOADED DURING INITAL PROGRAMMING


;Enable Global Interrupt
;========================
	sei


;Read ID from EEPROM (for Test only)  
;======================
	rcall		ReadIDFromEEPROM   ;read ID from EEPROM and copy to IDRegister
	;rcall		Delay1Sec
	;rcall		ReadIDFromEEPROMAndDisplay     ;FOR TEST ONLY WITH A SINGLE EMD, DO NOT DISPLAY ID NUMBER DURING SYSTEM 
	;                                           BOOTUP BECAUSE ALL UNITS WILL OVERLOAD 14V PS


;Initialize SPI Slave
;=====================
	ldi			temp, (1<<USIWM0)|(1<<USICS1)
	out			USICR, temp


;Run Staggered EMD Reset
;========================
;The nulls are staggered by 0.1 sec x ID number so the power supply doesn't overload
	rcall		StaggeredNull


;Flash LED Twice and Continue  (for testing only)
;=============================
	;rcall		FlashLED	 ;do not use when a"quick reset" is needed because flashing slows it way down
	;rcall		FlashLED  
	

;General Clears
;=================
	clr			PulseCounter		
	

;==============================================================================
;==============================================================================
;==============================================================================
;                         		       MAIN 
;==============================================================================
;==============================================================================
;==============================================================================

Main:
	rcall	CheckModePin	;configure Mode low = SPI, high(default) = SXS
	sbrs	Mode,0			;skip if Mode reg set
	rjmp	SPIMode			;
	rjmp	SXSMode			
	;	if Mode = low, enter loop listening to SPI pin, then display data
	;	if Mode = high(default), enter loop checking for SXS counts or reset

	rjmp	Main

;==============================================================================
;==============================================================================
;                         		     ROUTINES
;==============================================================================
;==============================================================================


;===========================================================================
;                 SPI MODE (RECEIVE AND DISPLAY SPI DATA)
;===========================================================================

;NOTE: THIS MODE RECEIVES 1 BYTE OF SPI DATA WHICH INCLUDES THE ADDRESS(HIGH NIB)
;OF THE DISPLAY ASSEMBLY AND THE NUMBER TO BE DISPLAYED(LOW NIB)

SPIMode:
	rcall	ReadIDFromEEPROM
	rcall   Debounce 
	rcall	StaggeredNull
		
ListenForSPIAddressedToThisEMDandDisplayLoop:
;*********************************************
    
    ;------------------------------
	;CHECK FOR MODE CHANGE
	sbic	PINA, ModePinIn  ;if cleared keep going in SPI Mode
	rjmp	SXSMode
	;------------------------------

ReceiveSPIByte:

;Note: According to AVR manual, an SPI slave must also configure DO pin = OUTPUT and USCK pin = INPUT, see PINS AND PORTS
;In the case of testing a slave Tiny2313 with a Arduino Micro(Mega) master:
;   Connect Arduino SCK to Tiny pin 19, UCSK
;   Connect Arduino MOSI(master out) to Tiny pin 17, MOSI/DI(data in)
;	Connect common ground to both devices

Init:
	ldi		temp, (1<<USIWM0)|(1<<USICS1)
	out		USICR, temp			;they are using "out" because it is putting bits in a special port register
SlaveSPITransfer:
	out		USIDR, temp
	ldi		temp, (1<<USIOIF)
	out		USISR, temp
SlaveSPITransfer_loop:
	sbis	USISR, USIOIF
	rjmp	SlaveSPITransfer_loop
	in		RcvdSPIByte, USIDR	   ;RcvdSPIByte = 4 bits of addr and 4 bits of data
	;ret    ;old

	;Test hi nib for ID match
		;if yes, send lo nib to display
		;if no, return
	mov		temp, RcvdSPIByte
	andi	temp, 0b11110000	 ;zero out lo nib	
	swap	temp				 ;swap nibs
	;compare to ID number
	cp		temp, IDRegister
	brne	IDNoMatch
	;ID matches, send lo SPI nib to display
	mov		temp, RcvdSPIByte
	andi	temp, 0b00001111	;zero out hi nib
	;temp = received data (0-15)

	;-----------------------------------------------------------------------------------------------------
	;if temp = 15, then read ID(1-13) and display on EMD for 2 secs then null: NOT NORMALLY USED
	;cpi		temp, 15
	;brne		ContRSB
	;rcall		ReadIDFromEEPROMandDisplay   ;this is a special routine because the ID can be two digits
	;rjmp		DoneRSB
	;ContRSB:
	;-----------------------------------------------------------------------------------------------------

	;therefore temp = 0-9	
	mov		PulseCounter, temp
	rcall	DisplayFinalPulseCount

	;DoneRSB:
	IDNoMatch:
	rjmp	ListenForSPIAddressedToThisEMDandDisplayLoop
	

;===============================================END SPI MODE

	


;=================================================================================================================
;                              SXS MODE (COUNT AND DISPLAY DIAL PULSES)
;=================================================================================================================

SXSMode:			//ACCEPTS NEGATIVE SXS PULSES  
;  
;*******  *******  *******  ************ HIGH
;      |  |     |  |     |  | 
;      ****     ****     **** LOW
;
	rcall		Debounce
	rcall		StaggeredNull
	clr			PulseCounter			;need to reset PulseCounter each time Mode is changed
	clr			HighIntervalCounter

;NOTE: DURING THIS ROUTINE WE NEED TO CHECK FOR A CHANGE OF THE MODE. THE TELEPHONE SWITCHES WERE SO
;NOISY THAT AN INTERRUPT ROUTINE WAS CONSTANTLY GETTING TRIGGERED, SO I GAVE UP ON IT AND RELIED ON
;CHECKING THE MODE PIN INSIDE OF EACH TIGHT LOOP. I RECOMMEND GOING BACK TO AN INTERRUPT IF THE 
;ENVIRONMENT IS NOT AS NOISY.

SXSStart1Loop:
;---------------------------
sbis	PINA, ModePinIn				;if ModePin is set then keep going
rjmp	SPIMode			
;---------------------------
;Tight Loop Test for low SXS pulse
sbic	PINB, SXSCountIn			;tight loop, skip next command if pulse is low  (responds to neg input pulse from 74LS14)
rjmp	SXSStart1Loop				;waiting for next low pulse
inc		PulseCounter				;if low, inc counter
rjmp	HighTestingLoop1			;pulse is low, go down to wait until it goes high
;************************************************
HighTestingLoop1:
sbis	PINB, SXSCountIn			;tight loop, waiting for low pulse to go high   
rjmp	HighTestingLoop1			
rcall	TwoMsDelay					;2ms delay
inc		HighIntervalCounter			;start adding up 2ms intervals that pulse has been high
;Check HighIntervalCounter Value
cpi		HighIntervalCounter, 75		;2ms x 75 = 150ms
brlo	PulseCountIsLow1			;branch if count < 75 
rcall	DisplayFinalPulseCount		;therefore count >= 75 so send count to EMD
clr		PulseCounter
clr		HighIntervalCounter
;************************************************
;Test for SXS LF Release pulse
;;;rjmp	SkipLFRelease1				;test only
SXSLFReleasePulseTestingLoop1:
sbis	PINB, SXSReleaseIn			;responds to neg input pulse from 74LS14
rjmp	ContLFR						;if Release pulse, then jmp to null
;-------------------------
   sbis	PINA, ModePinIn
   rjmp	SPIMode
   ;continue SXSMode
;-------------------------
sbic    PINB, SXSReleaseIn			 ;THIS IS INSIDE A LOOP THAT TESTS FOR RESET BUTTON INPUT, NEWLY ADDED 11/26/17
rjmp	SXSLFReleasePulseTestingLoop1  ;tight loop waiting to null displays before starting SXS switch dialing again
;*************************************************
ContLFR:
rcall	null
;------------------------
sbis	PINA, ModePinIn
rjmp	SPIMode
;------------------------
rjmp	SXSStart1Loop				;dial count has been displayed, go back to Start
PulseCountIsLow1:					;count is < 75 therefore keep counting high intervals, but also watch for another regular low pulse
sbic	PINB, SXSCountIn			;check for low   (responds to neg input pulse from 74LS14)
;------------------------
   ;sbis	PINA, ModePinIn         ;not needed here, I think
   ;rjmp	SPIMode
;------------------------
rjmp	HighTestingLoop1			;still high, keep counting high intervals 
;*************************************************
clr		HighIntervalCounter			;a low has occurred, it's time to jump back to the top and log it in by inc the PulseCounter
rjmp	SXSStart1Loop				;loop back up and wait for next dial pulse

;=====================================================END SXS MODE



;=======================================================================================================================
;                                            OTHER SUPPORT ROUTINES
;=======================================================================================================================

;Display Pulse Counter value on EM diplays
;==========================================
DisplayFinalPulseCount:
    mov		temp2, PulseCounter

	;TestForZero
	cpi		temp2, 0
	brne	TestForOne
	rcall	null
	ret
	TestForOne:
	cpi		temp2, 1
	brne	TestForTwo
	rcall	one
	ret
	TestForTwo:
	cpi		temp2, 2
	brne	TestForThree
	rcall	two
	ret
	TestForThree:
	cpi		temp2, 3
	brne	TestForFour
	rcall	three
	ret
	TestForFour:
	cpi		temp2, 4
	brne	TestForFive
	rcall	four
	ret
	TestForFive:
	cpi		temp2, 5
	brne	TestForSix
	rcall	five
	ret
	TestForSix:
	cpi		temp2, 6
	brne	TestForSeven
	rcall	six
	ret
	TestForSeven:
	cpi		temp2, 7
	brne	TestForEight
	rcall	seven
	ret
	TestForEight:
	cpi		temp2, 8
	brne	TestForNine
	rcall	eight
	ret
	TestForNine:
	cpi		temp2, 9
	brne	TestForTen
	rcall	nine
	ret
	TestForTen:
	rcall	zero
	ret


;Set the output ports to the segment drivers for specific decimal digit to display
;===================================================================================
one:
	ldi		temp, (0<<SegAOut)|(1<<SegBOut)|(1<<SegCOut)|(0<<SegDOut)|(0<<SegEOut)|(0<<SegFOut)|(0<<SegGOut)
	out		PortD, temp
	rcall	PulseSegments
	ret
two:
	ldi		temp, (1<<SegAOut)|(1<<SegBOut)|(0<<SegCOut)|(1<<SegDOut)|(1<<SegEOut)|(0<<SegFOut)|(1<<SegGOut)
	out		PortD, temp
	rcall	PulseSegments
	ret
three:
	ldi		temp, (1<<SegAOut)|(1<<SegBOut)|(1<<SegCOut)|(1<<SegDOut)|(0<<SegEOut)|(0<<SegFOut)|(1<<SegGOut)
	out		PortD, temp
	rcall	PulseSegments
	ret
four:
	ldi		temp, (0<<SegAOut)|(1<<SegBOut)|(1<<SegCOut)|(0<<SegDOut)|(0<<SegEOut)|(1<<SegFOut)|(1<<SegGOut)
	out		PortD, temp
	rcall	PulseSegments
	ret
five:
	ldi		temp, (1<<SegAOut)|(0<<SegBOut)|(1<<SegCOut)|(1<<SegDOut)|(0<<SegEOut)|(1<<SegFOut)|(1<<SegGOut)
	out		PortD, temp
	rcall	PulseSegments
	ret
six:
	ldi		temp, (1<<SegAOut)|(0<<SegBOut)|(1<<SegCOut)|(1<<SegDOut)|(1<<SegEOut)|(1<<SegFOut)|(1<<SegGOut)
	out		PortD, temp
	rcall	PulseSegments
	ret
seven:
	ldi		temp, (1<<SegAOut)|(1<<SegBOut)|(1<<SegCOut)|(0<<SegDOut)|(0<<SegEOut)|(0<<SegFOut)|(0<<SegGOut)
	out		PortD, temp
	rcall	PulseSegments
	ret
eight:
	ldi		temp, (1<<SegAOut)|(1<<SegBOut)|(1<<SegCOut)|(1<<SegDOut)|(1<<SegEOut)|(1<<SegFOut)|(1<<SegGOut)
	out		PortD, temp
	rcall	PulseSegments
	ret
nine:
	ldi		temp, (1<<SegAOut)|(1<<SegBOut)|(1<<SegCOut)|(1<<SegDOut)|(0<<SegEOut)|(1<<SegFOut)|(1<<SegGOut)
	out		PortD, temp
	rcall	PulseSegments
	ret
zero:
	ldi		temp, (1<<SegAOut)|(1<<SegBOut)|(1<<SegCOut)|(1<<SegDOut)|(1<<SegEOut)|(1<<SegFOut)|(0<<SegGOut)
	out		PortD, temp
	rcall	PulseSegments
	ret
null:
	ldi		temp, (0<<SegAOut)|(0<<SegBOut)|(0<<SegCOut)|(0<<SegDOut)|(0<<SegEOut)|(0<<SegFOut)|(0<<SegGOut)
	out		PortD, temp
	rcall	PulseSegments
	ret
smalli:		;for ID number display
	ldi		temp, (0<<SegAOut)|(0<<SegBOut)|(1<<SegCOut)|(0<<SegDOut)|(0<<SegEOut)|(0<<SegFOut)|(0<<SegGOut)
	out		PortD, temp
	rcall	PulseSegments
	ret
smalld:
	ldi		temp, (0<<SegAOut)|(1<<SegBOut)|(1<<SegCOut)|(1<<SegDOut)|(1<<SegEOut)|(0<<SegFOut)|(1<<SegGOut)
	out		PortD, temp
	rcall	PulseSegments
	ret

;==========================================
PulseSegments:
	rcall	TwoMsDelay			; let previous segment data sent to drivers settle for 2ms
	sbi		PortA, segPulseOut
	rcall	SegmentPulse		; 25 ms pulse to STBY pins of drivers
	cbi		PortA, segPulseOut
	ret

;================================================
SegmentPulse:
	clr		Counter0
	ldi		temp, SegmentPulseWidth		; typ: 25ms
	mov		Counter1, temp
	clr		temp
	SPLoop:
	dec		Counter0
	cpse	Counter0, temp	
	rjmp	SPLoop
	dec		Counter1
	cpse	Counter1, temp
	rjmp	SPLoop
	ret

;========================================
WriteIDtoEEPROM:
; ID number must be loaded in temp1 before calling this routine
	cli
	sbic	EECR, EEPE
	rjmp	WriteIDtoEEPROM
	ldi		temp, 1			;EEPROM address to write to
	out		EEAR, temp		;set up address
    ;ldi	temp, temp1		;EEPROM ID number     
	out		EEDR, temp1		;set up ID number
	sbi		EECR, EEMPE
	sbi		EECR, EEPE
	sei
	ret

;===========================================================================
ReadIDFromEEPROMandDisplay:
    rcall   ReadIDFromEEPROM

;Example display: i d 0 4
	
	TestTwoDigitIDDisplay:
;Display i then d
	rcall	smalli
	rcall	Delay1Sec		
	rcall	smalld
	rcall	Delay1Sec
;Display 2 digit ID number
	cpi		IDRegister, 10
	brsh	TenOrGreater
;Display 0 then 1-9
	ldi		PulseCounter, 10    ;a 10 displays as a 0, a 0 displays as a null
	rcall	DisplayFinalPulseCount
	;rcall	Delay1Sec
	;rcall	null
	rcall	Delay1Sec
	mov		PulseCounter, IDRegister
	rcall	DisplayFinalPulseCount
	rjmp	DoneIDD
;Display 1, then 0-3
	TenOrGreater:
	ldi		PulseCounter, 1		;display a 1
	rcall	DisplayFinalPulseCount
	rcall	Delay1Sec
	rcall	null
	rcall	Delay1Sec
	mov		temp, IDRegister
	subi	temp, 10
	;if 0 then change to a 10 because 0 is a null
	cpi		temp, 0
	brne	ContTOG
	ldi		temp, 10
	ContTOG:	
	mov		PulseCounter, temp
	rcall	DisplayFinalPulseCount
	DoneIDD:
	rcall	Delay1Sec
	rcall	null
	ret

;================================================================
ReadIDFromEEPROM:
	sbic	EECR, EEPE
	rjmp	ReadIDFromEEPROM
	ldi		temp, 1				;EEPROM address to read from
	out		EEAR, temp
	sbi		EECR, EERE
	in		IDRegister, EEDR	;read ID number 
	ret

;===============
TwoMsDelay:
	clr		Counter0
	ldi		temp, TwoMsDelayValue			; typ: 2ms
	mov		Counter1, temp
	clr		temp
	TMDLoop:
	dec		Counter0
	cpse	Counter0, temp	
	rjmp	TMDLoop
	dec		Counter1
	cpse	Counter1, temp
	rjmp	TMDLoop
	ret

;=============================================================
Debounce:
;Button debounce for both push and release of buttons used during test
;Note: Tactile buttons have most bouncing on release
	clr		Counter0
	ldi		temp, DebounceTime			; typ: 25ms
	mov		Counter1, temp
	clr		temp
DEBLoop:
	dec		Counter0
	cpse	Counter0, temp	
	rjmp	DEBLoop
	dec		Counter1
	cpse	Counter1, temp
	rjmp	DEBLoop
	ret

;==============================================================
;NOTE: This routine staggers the reset of multiple EMDs so the 14.5V power
;supply is not overloaded during a system reboot or Quick EMD reset
StaggeredNull:
	clr		temp2					;load temp2 with 0	
	mov		temp1, IDRegister
StaggerLoop:
	rcall	Debounce				;25ms x 4 = 100ms = 0.1 sec
	rcall	Debounce
	rcall	Debounce
	rcall	Debounce
	dec		temp1
	cpse	temp2, temp1
	rjmp	StaggerLoop	
	sbi		PortB, TestLED
	rcall	Debounce
	rcall	null
	cbi		PortB, TestLED
	ret

;====================================================================
CheckModePin:
;low = clr = SPI Mode
;high  = set = SXS Mode
	clr		Mode				;preclear Mode register
	sbic	PINA, ModePinIn		;if ModePin is already cleared (SPI mode) then skip setting it (SXS mode) 
	ser		Mode
	ret
	
;====================================================================
FlashLED:
	sbi		PortB, TestLED
	rcall	Delay1_4Sec
	cbi		PortB, TestLED
	rcall	Delay1_4Sec
	ret

;==============================================
;	       LONG INTERVAL LOOP TIMERS
;==============================================
Delay1_4Sec:
	ldi		temp, 1
	mov		Counter2, temp
	rjmp	ContDLY
Delay1_2Sec:
	ldi		temp, 2
	mov		Counter2, temp
	rjmp	ContDLY
Delay1Sec:
	ldi		temp, 4
	mov 	Counter2, temp
ContDLY:
	clr		temp
	mov		Counter0, temp
	mov		Counter1, temp
LongDelayLoop:
	dec		Counter0
	cpse	Counter0, temp	
	rjmp	LongDelayLoop
	dec		Counter1
	cpse	Counter1, temp
	rjmp	LongDelayLoop
	dec		Counter2
	cpse	Counter2, temp
	rjmp	LongDelayLoop
	ret


;END OF PROGRAM